;;; ===========================================================================
;;; ===========================================================================
;;; == CLASS:     RACECAR
;;; == FILE NAME: racecar.clp
;;; ==
;;; == SPECS:     This file contains the definitions for the "racear" class.
;;; ==            The following slots (attributes) are defined for the class
;;; ==            and thus for each racecar instance:
;;; ==
;;; ==            - Fuel capacity
;;; ==            - Fuel efficiency
;;; ==            - Damage level to nose chassis
;;; ==            - Fuel load
;;; ==            - Rain probability: Probability of rain
;;; ==            - Laps before entering pits
;;; ==            - How much fuel to pump
;;; ==            - Front chassis down-force
;;; ==            - Rear chassis down-force
;;; ==            - Tire types (4 tires total)
;;; ==
;;; ==            The following message handlers are defined:
;;; ==
;;; ==            - SetRacecarChassisNoseDamageLevel: Set chassis damage levels
;;; ==            - SetRacecarFuelLoad: Sets amount of fuel
;;; ==
;;; ===========================================================================
;;; ===========================================================================

;;; ===========================================================================
;;; ==                          CLASS DEFINITION
;;; ===========================================================================

(defclass RACECAR
	(is-a USER)
	(role concrete)

;;; ===========================================================================
;;; ==                               SLOTS
;;; ===========================================================================
;;; == The following slots will be defined at the time of declaration
;;; ===========================================================================

	(slot racecar_engine-fuel-capacity
		(type FLOAT))
		
	(slot racecar_engine-fuel-efficiency
		(type FLOAT))

;;; ===========================================================================
;;; == The following slots will be defined during execution
;;; ===========================================================================
		
	(slot racecar_chassis-nose-damage-level
		(type FLOAT)
		(range 0.0 5.0))

	(slot racecar_engine-fuel-load
		(type FLOAT))

;;; ===========================================================================
;;; == The following slots will be defined by rules and shown in the results
;;; ===========================================================================

	(slot racecar_laps-to-pit
		(type INTEGER))	

	(slot racecar_engine-fuel-to-pump
		(type FLOAT))

	(slot racecar_chassis-front-downforce
		(type SYMBOL)
		(allowed-symbols normal high low))

	(slot racecar_chassis-rear-downforce
		(type SYMBOL)
		(allowed-symbols normal high low))

	(slot racecar_left-front-tire
		(default-dynamic (make-instance (gensym*) of TIRE)))

	(slot racecar_right-front-tire
		(default-dynamic (make-instance (gensym*) of TIRE)))

	(slot racecar_left-rear-tire
		(default-dynamic (make-instance (gensym*) of TIRE)))

	(slot racecar_right-rear-tire
		(default-dynamic (make-instance (gensym*) of TIRE)))

)

;;; ===========================================================================
;;; ==                         MESSAGE HANDLERS
;;; ===========================================================================

(defmessage-handler RACECAR SetRacecarChassisNoseDamageLevel ()
	(printout t "Enter Chassis Nose Damage Level [0.0 - 5.0]: ") 
	(bind ?arg (read))
	(if (or (not (numberp ?arg)) (or (> ?arg 5.0) (< ?arg 0.0) ) )
		then 
			(printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
			(send ?self SetRacecarChassisNoseDamageLevel)
		else
			(send ?self put-racecar_chassis-nose-damage-level ?arg)
			(printout t "")
	)
)

(defmessage-handler RACECAR SetRacecarFuelLoad()
	(printout t "Enter Current Fuel Load [0.0 - " (send ?self get-racecar_engine-fuel-capacity) "]: ") 
	(bind ?arg (read))
	(if (or (not (numberp ?arg)) (or (> ?arg (send ?self get-racecar_engine-fuel-capacity)) (< ?arg 0.0) ) )
		then 
			(printout t "   -- Please enter a valid value. " ?arg " is NOT a valid value." crlf)
			(send ?self SetRacecarFuelLoad)
		else
			(send ?self put-racecar_engine-fuel-load ?arg)
			(printout t "")
	)
)

;;; ===========================================================================
;;; ===========================================================================
;;; ==                          END OF RACECAR CLASS
;;; ===========================================================================
;;; ===========================================================================	